@@ -22,7 +22,10 @@ sub mapTraverse {
 ## $oldval = CLASS->mapVisit($curval, \$code)
 sub mapVisit {
   my ($that,$nod,$code) = @_;
-  if (UNIVERSAL::isa($nod,'DDC::XS::Object')) {
+  if (
+      #UNIVERSAL::isa($nod,'DDC::XS::Object') ##-- breaks DDC::Any
+      ref($nod) && UNIVERSAL::can($nod,'members')
+     ) {
     my ($oldval,$newval);
     foreach my $slot (grep {$nod->can("get$_")} $nod->members) {
       $oldval = $nod->can("get${slot}")->($nod);
@@ -33,11 +36,13 @@ sub mapVisit {
     }
     return $code->($nod);
   }
-  elsif (UNIVERSAL::isa($nod,'ARRAY')) {
-    return [grep {defined($_)} map {$that->mapVisit($_,$code)} @$nod];
+  elsif (ref($nod) && UNIVERSAL::isa($nod,'ARRAY')) {
+    my $newval = [grep {defined($_)} map {$that->mapVisit($_,$code)} @$nod];
+    return ref($newval) eq 'ARRAY' ? $newval : bless($newval, ref($nod));
   }
-  elsif (UNIVERSAL::isa($nod,'HASH')) {
-    return {map {($_=>$that->mapVisit($nod->{$_},$code))} keys %$nod};
+  elsif (ref($nod) && UNIVERSAL::isa($nod,'HASH')) {
+    my $newval = {map {($_=>$that->mapVisit($nod->{$_},$code))} keys %$nod};
+    return ref($newval) eq 'HASH' ? $newval : bless($newval, ref($nod));
   }
   return $nod;
 }
@@ -49,11 +54,16 @@ sub mapVisit {
 ##--------------------------------------------------------------
 ## \%hash = $obj->toHash()
 ##  + returns an object as a (nested) perl hash
+##  + %opts:
+##    (
+##     trimClassNames => $bool,  ##-- auto-trim class-names?
+##     json => $bool,            ##-- for JSON-ification?
+##    )
 sub toHash {
-  my $obj = shift;
+  my ($obj,%opts) = @_;
   return $obj if (!defined($obj) && !ref($obj));
   my $class   = ref($obj);
-  #$class =~ s/^DDC::XS:://; ##-- use toJson()-style class names
+  $class =~ s/^DDC::(?:XS|PP|Any)::// if ($opts{trimClassNames} || $opts{json}); ##-- use toJson()-style class names
   return {
 	  (map {
 	    ( $_ => $obj->valToPerl($obj->can("get$_")->($obj)) )
@@ -126,7 +136,7 @@ sub newFromHash {
     if (!UNIVERSAL::isa($hash,'HASH'));
 
   $class = $hash->{class} if (defined($hash->{class}));
-  $class = "DDC::XS::$class" if (!UNIVERSAL::isa($class,__PACKAGE__)); ##-- honor toJson()-style class names
+  $class = "DDC::XS::$class" if ($class !~ /:/); ##-- honor toJson()-style class names
   my $obj = $class->new()
     or confess(__PACKAGE__, "::newFromHash(): $class->new() failed");
 
@@ -185,7 +195,7 @@ sub newFromJson {
 
 ## $json = $obj->TO_JSON
 sub TO_JSON {
-  return $_[0]->toHash();
+  return $_[0]->toHash(json=>1);
 }
 
 
